// TrackTrek '97 by Steven Lawrance
// Database Object

package sl.TrackTrek;

import java.io.*;

public abstract class DBObject extends Object
{
  InputStream parms;
  public final static int DATE = 1;
  public final static int DIST = 2;
  public final static int STR = 3;
  public final static int TIME = 4;
  public final static int EVENT = 5;
  public final static int TEAM = 6;
  public final static int TRACKMEET = 7;
  public final static int PARTICIPANT = 8;
  public final static int USER = 9;
  public final static int GROUP = 10;

  public DBObject(InputStream params)
  {
    this.parms = parms;
  }

  public DBObject()
  {
  }

  // Server requests
  public abstract void read(OutputStream out);
  public abstract void write(OutputStream out);

  // Initialization/Storage
  public abstract void load(InputStream in);
  public abstract void save(OutputStream out);

  // Convert object into a string
  public abstract String toString();

  // Comparison function
  public abstract int compareTo(DBObject other);

  // Value setters/getters
/*  public abstract void setValue(long value);
  public abstract void setValue(String value);
  public abstract long getLongValue();
  public abstract String getStringValue();*/

  // Sorting algorithm
  public static void sort(DBObject array[], int start, int end, boolean ascending)
  {
    // Sorts from start to end in the array
    int index, inner, r;
    DBObject temp;

    if (ascending)
    {
      // Sort in the ascending order
      for (r = start; r <= end - 1; r++)
      {
        index = r;
        for (inner = r + 1; inner <= end; inner++)
          if (array[inner].compareTo(array[index]) < 0)
            index = inner;
        if (index != r)
        {
          temp = array[index];
          array[index] = array[r];
          array[r] = temp;
        }
      }
    }
    else
    {
      // Sort in the decending order
      for (r = start; r <= end - 1; r++)
      {
        index = r;
        for (inner = r + 1; inner <= end; inner++)
          if (array[inner].compareTo(array[index]) > 0)
            index = inner;
        if (index != r)
        {
          temp = array[index];
          array[index] = array[r];
          array[r] = temp;
        }
      }
    }
  }

  // Other sorting algorithm
  public static void sort(DBObject array[], int numItems, boolean ascending)
  {
    // Call the sorting procedure to sort from the beginning to the end
    sort(array, 0, numItems - 1, ascending);
  }
}
