// TrackTrek '97 by Steven Lawrance
// Graphical Client

// This program allows people to use the information

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

// Client versions of the database objects

abstract class DBObject extends Frame
{
  public final static int OK = 1;
  public final static int ERROR = 2;
  public final static int PING = 3;
  public final static int READ = 4;
  public final static int WRITE = 5;
  public final static int CLOSE = 6;
  public final static int LOCK = 7;
  public final static int NOLOCK = 9;
  public final static int NOTEXIST = 10;
  public final static int UNLOCK = 11;
  public final static int ADD = 14;
  public final static int RENAME = 15;
  public final static int DELETE = 16;

  // unimplemented (user access messages; right now, everyone has full access)
  public final static int DENIED = 20;
  public final static int AUTH = 25;

  public final static int DATE = 1;
  public final static int DIST = 2;
  public final static int STR = 3;
  public final static int TIME = 4;
  public final static int EVENT = 5;
  public final static int TEAM = 6;
  public final static int TRACKMEET = 7;
  public final static int PARTICIPANT = 8;
  public final static int USER = 9;
  public final static int GROUP = 10;
  public final static int ROOT = 11;
  public final static int FILE = 12;

  // The object with global variables
  TTClient client;

  // Our location, so that we can load ourselves
  public String location;

  // Application flag (for property pages)
  public boolean applyToAll;

  public DBObject(String name, TTClient client)
  {
	super(name);
	this.client = client;
  }

  // Get this object's page
//  public abstract Panel getPanel();

  // Page type of this particular menu item
  public int pageType;

  // Initialization/Storage
  public abstract void load(DataInputStream in, boolean includeChildren);
  public abstract void save(DataOutputStream out, boolean includeChildren);

  // User-Interface related functions
  public abstract boolean refresh();

  // Data type
  public abstract int dataType();

}

class ttDist extends DBObject
{
  // Values for this object
  private long english;  // in 1/16 inches
  private long metric;   // in millimeters
  public boolean useMetric;
  public int trial;
  public int points;
  public int lane;

  // Display flags
  public boolean includeHuge;   // kilometers and miles
  public boolean includeLarge;  // meters and yards
  public boolean includeMedium; // decimeters and feet
  public boolean includeSmall;  // centimeters and inches
  public boolean includeTiny;   // millimeters and fractions of inches
  public boolean asFraction;    // whether or not to display as a fraction

  public ttDist()
  {
    super();
  }

  public int dataType()
  {
    return DIST;
  }

//  public abstract Panel getPanel()
//  {
    // Create a properties page
    //Panel 
//  }

  public void load(DataInputStream in, boolean includeChildren)
  {
    try {
      metric = in.readLong();
      english = in.readLong();
      useMetric = in.readBoolean();
      trial = in.readInt();
      points = in.readInt();
      lane = in.readInt();
      includeHuge = in.readBoolean();
      includeLarge = in.readBoolean();
      includeMedium = in.readBoolean();
      includeSmall = in.readBoolean();
      includeTiny = in.readBoolean();
      asFraction = in.readBoolean();
    }
    catch (IOException str) {}
  }

  public void save(DataOutputStream out, boolean includeChildren)
  {
    try {
      out.writeLong(metric);
      out.writeLong(english);
      out.writeBoolean(useMetric);
      out.writeInt(trial);
      out.writeInt(points);
      out.writeInt(lane);
      out.writeBoolean(includeHuge);
      out.writeBoolean(includeLarge);
      out.writeBoolean(includeMedium);
      out.writeBoolean(includeSmall);
      out.writeBoolean(includeTiny);
      out.writeBoolean(asFraction);
    }
    catch (IOException str) {}
  }

  public long getValue(boolean getMetric)
  {
    // Return the desired variable
    return (getMetric)? metric: english;
  }

  public void setValue(long distance, boolean useMetric)
  {
    // Set this distance's value to the given distance and set the current system
    this.useMetric = useMetric;
    this.metric = (useMetric)? distance: metricValue(distance);
    this.english = (useMetric)? englishValue(distance): distance;
  }

  public void setValue(long distance)
  {
    // Set this distance's value to the given distance in the current system
    this.metric = (useMetric)? distance: metricValue(distance);
    this.english = (useMetric)? englishValue(distance): distance;
  }

  public long englishValue(long metric)
  {
    // Convert millimeters into 1/16 inches
    return (long) (metric / 1.5875);
  }

  public long metricValue(long english)
  {
    // Convert 1/16 inches into millimeters
    return (long) (english * 1.5875);
  }

  public String toString()
  {
    // Convert myself into a string
    if (useMetric)
    {
      // Display with the metric system
      String output = new String();
      long distance = metric;

      if (includeHuge)
      {
        output = output.concat(java.lang.String.valueOf(distance / 1000000) + "km ");
        distance = distance % 1000000;
      }

      if (includeLarge)
      {
        output =  output.concat(java.lang.String.valueOf(distance / 1000) + "m ");
        distance = distance % 1000;
      }

      if (includeMedium)
      {
        output = output.concat(java.lang.String.valueOf(distance / 100) + "dm ");
        distance = distance % 100;
      }

      if (includeSmall)
      {
        output = output.concat(java.lang.String.valueOf(distance / 10) + "cm ");
        distance = distance % 10;
      }

      if (includeTiny)
        output = output.concat(java.lang.String.valueOf(distance) + "mm");

      // Remove the ending space (if any)
      output = output.trim();

      return output;
    }
    else
    {
      // Display with the english system
      String output = new String();
      long distance = english;

      if (includeHuge)
      {
        output = output.concat(java.lang.String.valueOf(distance / 1013760) + "mi ");
        distance = distance % 1013760;
      }

      if (includeLarge)
      {
        output = output.concat(java.lang.String.valueOf(distance / 576) + "yd ");
        distance = distance % 576;
      }

      if (includeMedium)
      {
        output = output.concat(java.lang.String.valueOf(distance / 192) + "'");
        distance = distance % 192;
      }

      if (includeSmall)
      {
        output = output.concat(java.lang.String.valueOf(distance / 16) + ((!includeTiny)? "\"": ""));
        distance = distance % 16;
      }

      if (includeTiny)
      {
        if (distance != 0)
        {
          if (asFraction)
          {
            // Write a simplified fraction
            if (distance % 8 == 0)
              output = output.concat(" 1/2\"");
            else
              if (distance % 4 == 0)
                output = output.concat(" " + java.lang.String.valueOf(distance / 4) + "/4\"");
              else
                if (distance % 2 == 0)
                  output = output.concat(" " + java.lang.String.valueOf(distance / 2) + "/8\"");
                else
                  output = output.concat(" " + java.lang.String.valueOf(distance) + "/16\"");
          }
          else
          {
            // Write as a decimal
            output = output.concat("." + java.lang.String.valueOf((long) ((distance * 100) / 16)));
          }
        }
        else
          if (includeSmall)
            output = output.concat("\"");
      }

      // Remove the ending space (if any)
      output = output.trim();

      return output;
    }
  } /* toString */

  public int compareTo(DBObject other)
  {
    // Get the number of minutes in both dates
    long otherValue = other.getLongValue();
    long thisValue = this.getLongValue();

    // This time is less than the other time
    if (otherValue > thisValue)
      return -1;

    // This time equals the other time
    if (otherValue == thisValue)
      return 0;

    // This time is greater than the other time
    return 1;
  }

  public String getStringValue()
  {
    return String.valueOf((useMetric)? metric: english);
  }

  public void setValue(String value)
  {
    long lValue = java.lang.Long.parseLong(value);
    setValue(lValue);
  }

  public long getLongValue()
  {
    // Return the desired variable in the current system
    return (useMetric)? metric: english;
  }
}

class ttStr extends DBObject
{
  // Value for this object
  public String value;

  public ttStr()
  {
    super();
  }

  public int dataType()
  {
    return STR;
  }

  public void load(DataInputStream in, boolean includeChildren)
  {
    try {
      // Load the string from the input
      value = in.readUTF();
    }
    catch (IOException str) {}
  }

  public void save(DataOutputStream out, boolean includeChildren)
  {
    try {
      // Save this string
      out.writeUTF(value);
    }
    catch (IOException str) {}
  }

  public String toString()
  {
    // Return ourselves :-)
    return new String(value);
  }

  public int compareTo(DBObject other)
  {
    // Get the number of minutes in both dates
    String otherValue = (other.getStringValue()).toLowerCase();
    String thisValue = value.toLowerCase();

    // Call the string's compareTo function and return that
    return thisValue.compareTo(otherValue);
  }

  public long getLongValue()
  {
    return Long.parseLong(value);
  }

  public String getStringValue()
  {
    return new String(this.value);
  }

  public void setValue(long value)
  {
    // Set the value
    this.value = (new Long(value)).toString();
  }

  public void setValue(String value)
  {
    this.value = new String(value);
  }
}

class ttTime extends DBObject
{
  // Values for this object
  public long value;
  public int trial;
  public int points;
  public int lane;

  // Display variables
  public boolean includeHours;
  public boolean includeMinutes;
  public boolean includeSeconds;
  public boolean includeCentiseconds;

  public ttTime()
  {
    super();
  }

  public int dataType()
  {
    return TIME;
  }

  public void load(DataInputStream in, boolean includeChildren)
  {
    try {
      value = in.readLong();
      trial = in.readInt();
      points = in.readInt();
      lane = in.readInt();
      includeHours = in.readBoolean();
      includeMinutes = in.readBoolean();
      includeSeconds = in.readBoolean();
      includeCentiseconds = in.readBoolean();
    }
    catch (IOException str) {}
  }

  public void save(DataOutputStream out, boolean includeChildren)
  {
    try {
      out.writeLong(value);
      out.writeInt(trial);
      out.writeInt(points);
      out.writeInt(lane);
      out.writeBoolean(includeHours);
      out.writeBoolean(includeMinutes);
      out.writeBoolean(includeSeconds);
      out.writeBoolean(includeCentiseconds);
    }
    catch (IOException str) {}
  }

  public String toString()
  {
    // Convert myself into a string
    String outp = new String();

    long time = this.value;
    if (includeHours)
    {
      outp = outp.concat(java.lang.String.valueOf((long) (time / 360000)) + ((includeMinutes)? ":": "hrs "));
      time = time % 360000;
    }

    if (includeMinutes)
    {
      outp = outp.concat((((time / 6000 < 10) && (includeHours))? "0": "") + java.lang.String.valueOf((long) (time / 6000)) + ((includeSeconds)? ":": "min "));
      time = time % 6000;
    }

    if (includeSeconds)
    {
      outp = outp.concat((((time / 100 < 10) && (includeMinutes))? "0": "") + java.lang.String.valueOf((long) (time / 100)));
      time = time % 100;
    }

    if (includeCentiseconds)
      outp = outp.concat("." + (((time < 10) && (includeSeconds))? "0": "") + java.lang.String.valueOf(time));

    // Remove the ending space (if any)
    outp = outp.trim();

    return outp;
  }

  public int compareTo(DBObject other)
  {
    // Get the number of minutes in both dates
    long otherValue = other.getLongValue();
    long thisValue = this.getLongValue();

    // This time is less than the other time
    if (otherValue > thisValue)
      return -1;

    // This time equals the other time
    if (otherValue == thisValue)
      return 0;

    // This time is greater than the other time
    return 1;
  }

  public long getLongValue()
  {
    return value;
  }

  public String getStringValue()
  {
    return String.valueOf(value);
  }

  public void setValue(long value)
  {
    // Set the value
    this.value = value;
  }

  public void setValue(String value)
  {
    this.value = java.lang.Long.parseLong(value);
  }
}

class ttEvent extends DBObject
{
  public final static int FIELDEVENT = 1;
  public final static int TRACKEVENT = 2;
  public final static int HEATSHEET = 3;
  public final static int RESERVED1 = 4;
  public final static int RESERVED2 = 5;

  public int scoreType;
  public Hashtable scores;
  public ttStr description;
  public ttDate day;
  public DBObject qualify;
  public int scoreIncrement;
  public int scoreHighest;
  public int scoreLowest;
  public int type;
  public boolean inTeamTotal;
  public boolean done;

  public ttEvent()
  {
    super();
    scores = new Hashtable();
  }

  public int dataType()
  {
    return EVENT;
  }

  public void load(DataInputStream in, boolean includeChildren)
  {
    try
    {
      // Clear the hashtable
      scores.clear();

      // Load the scores
      int i, numScores;
      numScores = in.readInt();
      scoreType = in.readInt();
      for (i = 1; i <= numScores; i++)
      {
        // Get this participant's name
        ttStr str = new ttStr();
        DBObject obj;
        str.load(in, includeChildren);

        if (includeChildren)
        {
          // Create the object
          switch(scoreType)
          {
            case DIST : obj = new ttDist(); break;
            case TIME : obj = new ttTime(); break;
          }

          // Load the object
          obj.load(in, includeChildren);

          // Add this score to the hashtable
          scores.put(str, obj);
        }
      }

      // Load the description and day
      description = new ttStr();
      description.load(in, includeChildren);
      day = new ttDate();
      day.load(in, includeChildren);

      // Load the qualifying object (for heat sheets)
      switch(scoreType)
      {
        case DIST : qualify = new ttDist(); break;
        case TIME : qualify = new ttTime(); break;
      }
      qualify.load(in, includeChildren);

      // Load the event's information
      scoreIncrement = in.readInt();
      scoreHighest = in.readInt();
      scoreLowest = in.readInt();
      type = in.readInt();
      inTeamTotal = in.readBoolean();
      done = in.readBoolean();
    } catch (IOException str) {  return; }
  }

  public void save(DataOutputStream out, boolean includeChildren)
  {
    try
    {
      // Save the score information
      int numScores = scores.size();
      out.writeInt(numScores);
      out.writeInt(scoreType);

      // Save the scores
      Enumeration list;
      for (list = scores.keys(); list.hasMoreElements();)
      {
        // Get this participant's name
        ttStr str = (ttStr) list.nextElement();
        str.save(out, includeChildren);

        if (includeChildren)
        {
          // Get the object and save it
          DBObject obj = (DBObject) scores.get(str);
          obj.save(out, includeChildren);
        }
      }

      // Save the description and day
      description.save(out, includeChildren);
      day.save(out, includeChildren);

      // Save the qualifying object (for heat sheets)
      qualify.save(out, includeChildren);

      // Save the event's information
      out.writeInt(scoreIncrement);
      out.writeInt(scoreHighest);
      out.writeInt(scoreLowest);
      out.writeInt(type);
      out.writeBoolean(inTeamTotal);
      out.writeBoolean(done);
    } catch (IOException str) {  return; }
  }
}

class Team extends DBObject
{
  public Vector events;
  public Hashtable participants;
  public ttStr description;
  public int teamTotal;

  public Team()
  {
    super();
    events = new Vector();
    participants = new Hashtable();
  }

  public int dataType()
  {
    return TEAM;
  }

  public void load(DataInputStream in, boolean includeChildren)
  {
    try
    {
      // Load the event pointers
      int i, numEvents;
      events.removeAllElements();
      numEvents = in.readInt();
      for (i = 1; i <= numEvents; i++)
      {
        // Get this event's value
        ttStr str = new ttStr();
        str.load(in, includeChildren);

        // Add this score to the vector
        events.addElement(str);
      }

      // Load the participants for this team
      int numParticipants;
      participants.clear();
      numParticipants = in.readInt();
      for (i = 1; i <= numParticipants; i++)
      {
        // Get this participant's name
        ttStr str = new ttStr();
        DBObject obj;
        str.load(in, includeChildren);

        if (includeChildren)
        {
          // Load the object
          obj = new Participant();
          obj.load(in, includeChildren);

          // Add this score to the hashtable
          participants.put(str, obj);
        }
      }

      // Load the description
      description = new ttStr();
      description.load(in, includeChildren);

      // Load the team's information
      teamTotal = in.readInt();
    } catch (IOException str) {  return; }
  }

  public void save(DataOutputStream out, boolean includeChildren)
  {
    try
    {
      // Save the event pointers
      int numEvents = events.size();
      out.writeInt(numEvents);
      Enumeration list;
      for (list = events.elements(); list.hasMoreElements();)
      {
        // Get this event's name
        ttStr str = (ttStr) list.nextElement();
        str.save(out, includeChildren);
      }

      // Save the participants
      int numParticipants = participants.size();
      out.writeInt(numParticipants);
      for (list = participants.keys(); list.hasMoreElements();)
      {
        // Get this participant's name
        ttStr str = (ttStr) list.nextElement();
        str.save(out, includeChildren);

        if (includeChildren)
        {
          // Get the object and save it
          DBObject obj = (DBObject) participants.get(str);
          obj.save(out, includeChildren);
        }
      }

      // Save the description
      description.save(out, includeChildren);

      // Save the event's information
      out.writeInt(teamTotal);
    } catch (IOException str) {  return; }
  }
}

class Participant extends DBObject
{
  public Vector events;
  public ttStr description;
  public int points;

  public Participant()
  {
	// Create the main window with our name on it
	super(name + " - TrackTrek '97");
	setBackground(new Color(192, 192, 192));
	this.name = name;

	// Add the menu bases
	MenuBar menuBar = new MenuBar();
	Menu menuParticipant = new Menu("Participant");

	// Add the menu items for the Participant menu
	menuTrackMeet.add(new MenuItem("New"));
	menuTrackMeet.add(new MenuItem("Open"));
	menuTrackMeet.add(new MenuItem("Delete"));
	menuTrackMeet.add(new MenuItem("-"));
	menuTrackMeet.add(new MenuItem("Properties"));
	menuTrackMeet.add(new MenuItem("-"));
	menuTrackMeet.add(new MenuItem("Close"));

	// Add these menus to the window
	menuBar.add(menuParticipant);
	setMenuBar(menuBar);

	// Create the view for this window
	GridBagLayout layout;
	GridBagConstraints gbc;
	setLayout(layout);
	gbc.gridwidth = gcb.REMAINDER;
	gbc.fill = gbc.BOTH;
	gbc.weightx  =  1.0;

	// Add the track meet view
	Label tmLabel = new Label("Track Meets:");
	tmList = new List();
	gbc.gridheight = 1;
	layout.setConstraints(tmLabel, gbc);
	add(tmLabel);
	gbc.gridheight = 5;
	layout.setConstraints(tmList, gbc);
	add(tmList);

	// Add the team view
	Label teamLabel = new Label("Teams:");
	teamList = new List();
	gbc.gridheight = 1;
	layout.setConstraints(teamLabel, gbc);
	add(teamLabel);
	gbc.gridheight = 5;
	layout.setConstraints(teamList, gbc);
	add(teamList);

	// Show this window
	show();

	// Initialize the variables
    events = new Vector();
  }

  public boolean action(Event  evt, Object  what)
  {
	if (evt.id == evt.ACTION_EVENT)
	{
	  if (what instanceof String)
	  {
		// The action event's "what" object is a string
	    String name = (String) what;

		if (evt.target instanceof MenuItem)
		{
		  // This is a menu item; use its name
		  if (name.equals("New"))
	}
  }

  public int dataType()
  {
    return PARTICIPANT;
  }

  public void load(DataInputStream in, boolean includeChildren)
  {
    try
    {
      // Load the event pointers
      int i, numEvents;
      numEvents = in.readInt();
      for (i = 1; i <= numEvents; i++)
      {
        // Get this event's value
        ttStr str = new ttStr();
        str.load(in, includeChildren);

        // Add this score to the vector
        events.addElement(str);
      }

      // Load the description
      description = new ttStr();
      description.load(in, includeChildren);

      // Load the participant's information
      points = in.readInt();
    } catch (IOException str) {  return; }
  }

  public void save(DataOutputStream out, boolean includeChildren)
  {
    try
    {
      // Save the event pointers
      int numEvents = events.size();
      out.writeInt(numEvents);
      Enumeration list;
      for (list = events.elements(); list.hasMoreElements();)
      {
        // Get this event's name
        ttStr str = (ttStr) list.nextElement();
        str.save(out, includeChildren);
      }

      // Save the description
      description.save(out, includeChildren);

      // Save the participant's information
      out.writeInt(points);
    } catch (IOException str) {  return; }
  }
}

class ttDate extends DBObject
{
  // Values for this object
  public int year;
  public int month;
  public int weekDay;
  public int day;
  public int hour;
  public int minute;
  public boolean includeWeekDay;
  public boolean includeTime;

  // String representations
  String monthStr[] = { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };
  String weekDayStr[] = { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" };

  public ttDate()
  {
    super("ttDate");
  }

  public int dataType()
  {
    return DATE;
  }

  public void load(DataInputStream in, boolean includeChildren)
  {
    try {
      // Load our variables from the input stream
      year = in.readInt();
      month = in.readInt();
      day = in.readInt();
      weekDay = in.readInt();
      hour = in.readInt();
      minute = in.readInt();
      includeWeekDay = in.readBoolean();
      includeTime = in.readBoolean();
    }
    catch (IOException str) {}
  }

  public void save(DataOutputStream out, boolean includeChildren)
  {
    try {
      // Save our variables into the output stream
      out.writeInt(year);
      out.writeInt(month);
      out.writeInt(day);
      out.writeInt(weekDay);
      out.writeInt(hour);
      out.writeInt(minute);
      out.writeBoolean(includeWeekDay);
      out.writeBoolean(includeTime);
    }
    catch (IOException str) {}
  }

  public String toString()
  {
    // Convert myself into a string
    return ((includeWeekDay)? weekDayStr[weekDay] + ", ": "") + monthStr[month] +
            " " + java.lang.String.valueOf(day + 1) + ", " + java.lang.String.valueOf(year) + 
            ((includeTime)? "  " + java.lang.String.valueOf((hour == 0)? 12: (hour + 1) % 12) + ":" + java.lang.String.valueOf(minute + 1) +
            ((hour >= 12)? "PM": "AM"): "");
  }

  public void setValue(long value)
  {
	// Set the values
    year = (int) value / (13 * 32 * 25 * 61);
    value = value % (13 * 32 * 25 * 61);

    month = (int) value / (32 * 25 * 61);
    value = value % (32 * 25 * 61);

    day = (int) value / (25 * 61);
    value = value % (25 * 61);

    hour = (int) value / 61;
    value = value % 61;

	minute = (int) value;


	// Determine the day of the week
	Date date = new Date(year - 1900, month, day + 1);
	weekDay = date.getDay();
/*  Java 1.1:  GregorianCalendar weekDay = new GregorianCalendar(year, month, day + 1);
    weekDay.complete();
    this.weekDay = weekDay.get(Calendar.DAYOFWEEK) - Calendar.SUNDAY;*/
  }

  public static long getSetValue(int year, int month, int day, int hour, int minute)
  {
     // Convert ourselves into minutes for the setValue() function
	 long minutes;
     minutes = year * 13 * 32 * 25 * 61;
     minutes += month * 32 * 25 * 61;
     minutes += day * 25 * 61;
     minutes += hour * 61;
	 minutes += minute;
	 return minutes;
  }

  public int compareTo(DBObject other)
  {
	 // Get the number of minutes in both dates
	 long otherValue = other.getLongValue();
	 long thisValue = this.getLongValue();

	 // This time is less than the other time
	 if (otherValue > thisValue)
	   return -1;

	 // This time equals the other time
	 if (otherValue == thisValue)
	   return 0;

	 // This time is greater than the other time
	 return 1;
  }

  public long getLongValue()
  {
	 // Convert ourselves into minutes; only use this result with DateType functions!!
	 long minutes;
	 minutes = year * 12 * 31 * 24 * 60;
	 minutes += month * 31 * 24 * 60;
	 minutes += day * 24 * 60;
	 minutes += hour * 60;
	 minutes += minute;
	 return minutes;
  }

  public String getStringValue()
  {
    return String.valueOf(getSetValue(year, month, day, hour, minute));
  }

  public void setValue(String value)
  {
	long lValue = java.lang.Long.parseLong(value);
	setValue(lValue);
  }
}

class TrackMeet extends DBObject
{
  public Hashtable events;
  public ttStr description;
  public String name;

  public TrackMeet(String name)
  {
	// Create the main window with our name on it
	super(name + " - TrackTrek '97");
	setBackground(new Color(192, 192, 192));
	this.name = name;

	// Add the menu bases
	MenuBar menuBar = new MenuBar();
	Menu menuTrackMeet = new Menu("&Track Meet");
	Menu menuEvent = new Menu("&Event");

	// Add the menu items for the Track Meet menu
	menuTrackMeet.add(new MenuItem("&New"));
	menuTrackMeet.add(new MenuItem("&Open"));
	menuTrackMeet.add(new MenuItem("&Delete"));
	menuTrackMeet.add(new MenuItem("-"));
	menuTrackMeet.add(new MenuItem("&Properties"));
	menuTrackMeet.add(new MenuItem("-"));
	menuTrackMeet.add(new MenuItem("&Close"));

	// Add the menu items for the Event menu
	menuEvent.add(new MenuItem("&New"));
	menuEvent.add(new MenuItem("&Open"));
	menuEvent.add(new MenuItem("&Delete"));
	menuEvent.add(new MenuItem("-"));
	menuEvent.add(new MenuItem("&Properties"));
	menuEvent.add(new MenuItem("-"));
	menuEvent.add(new MenuItem("&Import Heat Sheet"));

	// Add these menus to the window
	menuBar.add(menuTrackMeet);
	menuBar.add(menuEvent);
	setMenuBar(menuBar);

	// Create the view for this window
	GridBagLayout layout;
	GridBagConstraints gbc;
	setLayout(layout);
	gbc.gridwidth = gcb.REMAINDER;
	gbc.fill = gbc.BOTH;
	gbc.weightx  =  1.0;

	// Add the track meet view
	Label tmLabel = new Label("Track Meets:");
	tmList = new List();
	gbc.gridheight = 1;
	layout.setConstraints(tmLabel, gbc);
	add(tmLabel);
	gbc.gridheight = 5;
	layout.setConstraints(tmList, gbc);
	add(tmList);

	// Add the team view
	Label teamLabel = new Label("Teams:");
	teamList = new List();
	gbc.gridheight = 1;
	layout.setConstraints(teamLabel, gbc);
	add(teamLabel);
	gbc.gridheight = 5;
	layout.setConstraints(teamList, gbc);
	add(teamList);

	// Show this window
	show();

	// Initialize the variables
    events = new Hashtable();
  }

  public int dataType()
  {
    return TRACKMEET;
  }

  public void load(DataInputStream in, boolean includeChildren)
  {
    try
    {
      // Clear the hashtable
      events.clear();

      // Load the events
      int i, numEvents;
      numEvents = in.readInt();
      for (i = 1; i <= numEvents; i++)
      {
        // Get this event's name
        ttStr str = new ttStr();
        DBObject obj;
        str.load(in, includeChildren);

        if (includeChildren)
        {
          // Load the object
          obj = new ttEvent();
          obj.load(in, includeChildren);

          // Add this score to the hashtable
          events.put(str, obj);
        }
      }

      // Load the description
      description = new ttStr();
      description.load(in, includeChildren);

    } catch (IOException str) {  return; }
  }

  public void save(DataOutputStream out, boolean includeChildren)
  {
    try
    {
      // Save event information
      int numEvents = events.size();
      out.writeInt(numEvents);

      // Save the scores
      Enumeration list;
      for (list = events.keys(); list.hasMoreElements();)
      {
        // Get this event's name
        ttStr str = (ttStr) list.nextElement();
        str.save(out, includeChildren);

        if (includeChildren)
        {
          // Get the object and save it
          DBObject obj = (DBObject) events.get(str);
          obj.save(out, includeChildren);
        }
      }

      // Save the description
      description.save(out, includeChildren);

    } catch (IOException str) {  return; }
  }
}

class ttRoot extends DBObject
{
  public File hFile;
  public Hashtable teams;
  public Hashtable trackMeets;
  public ttStr description;
  public int saveFrequency;

  // Components for this window
  List teamList;
  List tmList;

  public ttRoot(TTClient client)
  {
	// Create the main window
	super("TrackTrek '97", client);
	setBackground(new Color(192, 192, 192));

	// Add the menu bases
	MenuBar menuBar = new MenuBar();
	Menu menuTrackMeet = new Menu("&Track Meet");
	Menu menuEvent = new Menu("&Team");

	// Add the menu items for the Track Meet menu
	menuTrackMeet.add(new MenuItem("&New"));
	menuTrackMeet.add(new MenuItem("&Open"));
	menuTrackMeet.add(new MenuItem("&Delete"));
	menuTrackMeet.add(new MenuItem("-"));
	menuTrackMeet.add(new MenuItem("&Properties"));
	menuTrackMeet.add(new MenuItem("-"));
	menuTrackMeet.add(new MenuItem("E&xit"));

	// Add the menu items for the Event menu
	menuEvent.add(new MenuItem("&New"));
	menuEvent.add(new MenuItem("&Open"));
	menuEvent.add(new MenuItem("&Delete"));
	menuEvent.add(new MenuItem("-"));
	menuEvent.add(new MenuItem("&Properties"));

	// Add these menus to the window
	menuBar.add(menuTrackMeet);
	menuBar.add(menuEvent);
	setMenuBar(menuBar);

	// Create the view for this window
	GridBagLayout layout;
	GridBagConstraints gbc;
	setLayout(layout);
	gbc.gridwidth = gcb.REMAINDER;
	gbc.fill = gbc.BOTH;
	gbc.weightx  =  1.0;

	// Add the track meet view
	Label tmLabel = new Label("Track Meets:");
	tmList = new List();
	gbc.gridheight = 1;
	layout.setConstraints(tmLabel, gbc);
	add(tmLabel);
	gbc.gridheight = 5;
	layout.setConstraints(tmList, gbc);
	add(tmList);

	// Add the team view
	Label teamLabel = new Label("Teams:");
	teamList = new List();
	gbc.gridheight = 1;
	layout.setConstraints(teamLabel, gbc);
	add(teamLabel);
	gbc.gridheight = 5;
	layout.setConstraints(teamList, gbc);
	add(teamList);

	// Show this window
	show();

	// Initialize the variables
    teams = new Hashtable();
    trackMeets = new Hashtable();
    description = new ttStr();
    description.value = "This is the root object";
    saveFrequency = 0;
  }

  public int dataType()
  {
    return ROOT;
  }

  public void load(DataInputStream in, boolean includeChildren)
  {
    try
    {
      // Clear the hashtables
      teams.clear();
      trackMeets.clear();

      // Load the teams
      int i, numTeams;
      numTeams = in.readInt();
      for (i = 1; i <= numTeams; i++)
      {
        // Get this team's name
        ttStr str = new ttStr();
        DBObject obj;
        str.load(in, includeChildren);

        if (includeChildren)
        {
          // Load the object
          obj = new Team();
          obj.load(in, includeChildren);

          // Add this score to the hashtable
          teams.put(str, obj);
        }
      }

      // Load the track meets
      int numTrackMeets;
      numTrackMeets = in.readInt();
      for (i = 1; i <= numTrackMeets; i++)
      {
        // Get this track meet's name
        ttStr str = new ttStr();
        DBObject obj;
        str.load(in, includeChildren);

        if (includeChildren)
        {
          // Load the object
          obj = new TrackMeet();
          obj.load(in, includeChildren);

          // Add this score to the hashtable
          trackMeets.put(str, obj);
        }
      }

      // Load the description
      description = new ttStr();
      description.load(in, includeChildren);

      // Load the root variables
      saveFrequency = in.readInt();
    } catch (IOException str) {  return; }
  }

  public boolean refresh()
  {
	// Connect to the server to get the list information
    try
    {
      // Create a new connection to the server
	  TTConn conn = new TTConn(this);
	  conn.connect();

	  // Obtain a read lock to the root
	  if (conn.lockForRead("", false) != LOCK)
	  {
		// Could not get access; close connection
		tmList.addItem("Could not read names!");
		teamList.addItem("Could not read names!");
		conn.close();
		return false;
	  }

	  // Read the root object
	  ttRoot root = new ttRoot();
	  conn.read("", root, false);

	  // Close the connection
	  conn.close();

	  // Display the names
	  root.displayList(tmList, teamList);
	  return true;
    }
    catch (IOException str)
    {
	  return false;
    }
  }

  public void save(DataOutputStream out, boolean includeChildren)
  {
    try
    {
      // Save team information
      int numTeams = teams.size();
      out.writeInt(numTeams);

      // Save the teams
      Enumeration list;
      for (list = teams.keys(); list.hasMoreElements();)
      {
        // Get this team's name
        ttStr str = (ttStr) list.nextElement();
        str.save(out, includeChildren);

        if (includeChildren)
        {
          // Get the object and save it
          DBObject obj = (DBObject) teams.get(str);
          obj.save(out, includeChildren);
        }
      }

      // Save track meet information
      int numTrackMeets = trackMeets.size();
      out.writeInt(numTrackMeets);

      // Save the track meets
      for (list = trackMeets.keys(); list.hasMoreElements();)
      {
        // Get this track meet's name
        ttStr str = (ttStr) list.nextElement();
        str.save(out, includeChildren);

        if (includeChildren)
        {
          // Get the object and save it
          DBObject obj = (DBObject) trackMeets.get(str);
          obj.save(out, includeChildren);
        }
      }

      // Save the description
      description.save(out, includeChildren);

      // Save the root variables
      out.writeInt(saveFrequency);
    } catch (IOException str) {  return; }
  }

  public boolean saveToFile()
  {
    try { try {
      // Initialize the streams
      FileOutputStream fileOut = new FileOutputStream(hFile);
      DataOutputStream out = new DataOutputStream(fileOut);

      // Write the header
      String header = "TrackTrek '97  version: 1.00.97a  program: Graphical Client (Java Version)";
      out.writeUTF(header);

      // Write an escape character
      out.write(27);

      // Save ourself to this file
      save(out, true);

      // Done saving :-)  -- Close the file
      fileOut.close();
    } catch (IOException str) { return false; } } catch (FileNotFoundException str) { return false; }

    // Everything was great!  Return true
    return true;
  }

  public boolean loadFromFile(String file)
  {
    // Open the file provided; open "Local Database.db" if none specified
    if (file == null)
      file = "Local Database.db";
    hFile = new File(file);

    try { try {
      // Initialize the streams
      FileInputStream fileIn = new FileInputStream(hFile);
      DataInputStream in = new DataInputStream(fileIn);

      // Check the header
      String header = in.readUTF();

      // If the header is not the first version type, return false
      if (header.startsWith("TrackTrek '97  version: 1.0"))
        return false;

      // Read the escape character
      in.read();

      // Load ourself from this file
      load(in, true);

      // Done reading :-)  -- Close the file, for now
      fileIn.close();
    } catch (IOException str) { return false; } } catch (FileNotFoundException str) { return false; }

    // Everything was great!  Return true
    return true;
  }
}


// Client Threads/Windows-----------------------------

class TTConn extends Object
{
  // Server message constants
  public final static int OK = 1;
  public final static int ERROR = 2;
  public final static int PING = 3;
  public final static int READ = 4;
  public final static int WRITE = 5;
  public final static int CLOSE = 6;
  public final static int LOCK = 7;
  public final static int NOLOCK = 9;
  public final static int NOTEXIST = 10;
  public final static int UNLOCK = 11;
  public final static int ADD = 14;
  public final static int RENAME = 15;
  public final static int DELETE = 16;

  // unimplemented (user access messages; right now, everyone has full access)
  public final static int DENIED = 20;
  public final static int AUTH = 25;

  // Server connection
  Socket conn;
  DataInputStream in;
  DataOutputStream out;

  // TTClient (for host name a port)
  TTClient client;

  public TTConn(TTClient client)
  {
	// Set the TTClient info, but don't connect yet..
	this.client = client;
  }

  public Socket connect()
  {
	try
	{
	  // Connect to the server
      conn = new Socket(host, port);
      in = new DataInputStream(conn.getInputStream());
      out = new DataOutputStream(conn.getOutputStream());
	  return conn;
	} catch (IOException str)
	{
	  return null;
	}
  }

  public void close()
  {
	try
	{
	  // Send a CLOSE message and wait for a response
	  out.writeInt(CLOSE);
	  in.readInt();
	} catch (IOException str)
	{
	}

	try
	{
	  // Disconnect from the server
      conn.close();
	} catch (IOException str)
	{
	}

	conn = null;
	in = null;
	out = null;
  }

  public int lockForRead(String location, boolean lockChildren)
  {
    // Lock for read access, waiting for at most 5 seconds
	return lock(true, 5000, location, lockChildren, false);
  }

  public int lock(boolean waitFlag, int waitDuration, String location, boolean lockChildren, boolean exclusive)
  {
	try
	{
	  // Make a database lock
	  out.writeInt(LOCK);
	  out.writeBoolean(waitFlag);
	  out.writeInt(waitDuration);
	  out.writeUTF(location);
	  out.writeBoolean(lockChildren);
	  out.writeBoolean(exclusive);

	  // Reaturn the result
	  return in.readInt();
	} catch (IOException str)
	{
	  return ERROR;
	}
  }
  
  public int unlock(String location, boolean unlockChildren)
  {
	try
	{
	  // Unlock a database lock
	  out.writeInt(UNLOCK);
	  out.writeUTF(location);
	  out.writeBoolean(unlockChildren);

	  // Reaturn the result
	  return in.readInt();
	} catch (IOException str)
	{
	  return ERROR;
	}
  }
  
  public String ping()
  {
	try
	{
      // Get the server version
	  boolean success = true;
	  out.writeInt(PING);
	  success = success && (in.readInt() == OK);
	  success = success && (in.readInt() == PING);
	  success = success && (in.readInt() == PING);
	  success = success && (in.readInt() == PING);
	  String ver = in.readUTF();
	  success = success && (in.readInt() == OK);
	
	  // Return either the version or the failure
	  if (success)
	    return ver;
	  else
	    return null;
	} catch (IOException str)
	{
	  return null;
	}
  }

  public int read(String location, DBObject data, boolean readChildren)
  {
	try
	{
	  // Request an object
	  out.writeInt(READ);
	  out.writeUTF(location);
	  out.writeBoolean(readChildren);

	  // Check the response
	  switch (in.readInt())
	  {
	    case OK :
		  // Object read ok, let's get the object
		  data.load(in, readChildren);
		  in.readInt();
		  return OK;

		case NOLOCK :
		  // Object has no lock!
		  return NOLOCK;

		case NOTEXIST :
		  // Object does not exist!
		  return NOTEXIST;

		default :
		  // Unrecognized response!
		  return ERROR;
	  }
	} catch (IOException str)
	{
	  return ERROR;
	}
  }

  public int write(String location, DBObject data, boolean writeChildren)
  {
	try
	{
	  // Request a write operation
	  out.writeInt(WRITE);
	  out.writeUTF(location);
	  out.writeBoolean(writeChildren);

	  // Check the response
	  switch (in.readInt())
	  {
	    case NOLOCK :
		  // Object is not locked in the server
		  return NOLOCK;

		case OK :
		  // Write operation ok, now I'll send the data
		  data.save(out, writeChildren);
		  out.writeInt(OK);
		  return OK;

		case NOTEXIST :
		  // Object does not exist
		  return NOTEXIST;

		default :
		  // Error: unrecognized response
		  return ERRROR;
	  }
	}
	catch (IOException str)
	{
	  // IOException error
	  return ERROR;
	}
  }

  public int delete(String parent, String name)
  {
	try
	{
	  // Delete a child
	  out.writeInt(DELETE);
	  out.writeUTF(parent);
	  out.writeUTF(name);

	  // Return the response
	  return in.readInt();
	}
	catch (IOException str)
	{
	  // IOException error
	  return ERROR;
	}
  }

  public int rename(String parent, String oldName, String newName)
  {
	try
	{
	  // Rename a child
	  out.writeInt(RENAME);
	  out.writeUTF(parent);
	  out.writeUTF(oldName);
	  out.writeUTF(newName);

	  // Return the response
	  return in.readInt();
	}
	catch (IOException str)
	{
	  // IOException error
	  return ERROR;
	}
  }

  public int add(String parent, String name, DBObject obj, boolean addChildren)
  {
	try
	{
	  // Add a child to a parent
	  out.writeInt(ADD);
	  out.writeUTF(parent);
	  out.writeUTF(name);
	  out.writeInt(obj.dataType());
	  out.writeBoolean(addChildren);

	  // Check the response
	  switch (in.readInt())
	  {
		case OK :
		  // Object added; send the data and OK
		  obj.save(out, addChildren);
		  out.writeInt(OK);
		  return OK;

		case NOTEXIST :
		  // Object does not exist
		  return NOTEXIST;

		default :
		  // Error: unrecognized response, or response was ERROR
		  return ERRROR;
	  }
	}
	catch (IOException str)
	{
	  // IOException error
	  return ERROR;
	}
  }

}

public class TTClient implements Runnable
{
  // Server information
  String host;
  int port;

  public void run()
  {
  }

  public void setServer(String host, int port)
  {
    this.host = host;
	this.port = port;
  }

  public static void main(String args[])
  {
    // Create ourselves and let the creation do the rest
	setServer("127.0.0.1", 3040);
	ttRoot root = new ttRoot(this);
	root.refresh();
//	TTClient client = new TTClient();
//	client.refresh();
  }
}
