// TrackTrek '97 by Steven Lawrance
// Ping Utility

import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import EventPropertiesDialog;

class PingButton extends Button
{
  public final static int OK = 1;
  public final static int ERROR = 2;
  public final static int PING = 3;
  public final static int READ = 4;
  public final static int WRITE = 5;
  public final static int CLOSE = 6;
  public final static int LOCK = 7;
  public final static int NOLOCK = 9;
  public final static int NOTEXIST = 10;
  public final static int UNLOCK = 11;

  Label status;
  TextField hostName;

  public PingButton(String name, Label status, TextField hostName)
  {
    super(name);
    this.status = status;
    this.hostName = hostName;
  }

  public boolean action(Event evt, Object what)
  {
    if (evt.id == Event.ACTION_EVENT)
    {
      // Ping the server

      try { try { try {
      // Make a connection to the local server
      String serverName = hostName.getText();
      int serverPort = 3040;
      int i = serverName.lastIndexOf(':');
      if (i != -1)
      {
        // Separate the port and the name from the string, if any
        serverPort = Integer.parseInt(serverName.substring(i + 1));
        serverName = serverName.substring(0, i);
      }
      status.setText("Contacting server " + serverName + "...");
      Socket conn = new Socket(serverName, serverPort);
      DataOutputStream out = new DataOutputStream(conn.getOutputStream());
      DataInputStream in = new DataInputStream(conn.getInputStream());

      // Send the ping
      out.writeInt(PING);
      status.setText("Ping sent, waiting for reply...");

      // Wait for reply
      String version;
      boolean success = true;
      success = success && (in.readInt() == OK);
      status.setText("Acknowledgement 1/2");
      Thread.sleep(100);
      success = success && (in.readInt() == PING);
      status.setText("Ping 1/3");
      Thread.sleep(100);
      success = success && (in.readInt() == PING);
      status.setText("Ping 2/3");
      Thread.sleep(100);
      success = success && (in.readInt() == PING);
      status.setText("Ping 3/3");
      Thread.sleep(100);
      version = in.readUTF();
      status.setText("Version sent");
      Thread.sleep(100);
      success = success && (in.readInt() == OK);
      status.setText("Acknowledgement 2/2");
      Thread.sleep(100);

      // Close connection
      out.writeInt(CLOSE);
      conn.close();

      // Display result
      String msg = "Reply " + ((success)? "good, ": "bad, ") + "version is \"" + version + "\"";
      status.setText(msg);
      } catch (UnknownHostException str) { status.setText("Unknown host"); } } catch (IOException str) { status.setText("Connection failure"); } } catch (InterruptedException str)
      {
        status.setText("Internal error with ping");
      }
    }
    return true;
  }
}

class ExitButton extends Button
{
  Label status;

  public ExitButton(String name, Label status)
  { 
    super(name);
    this.status = status;
  }

  public boolean action(Event evt, Object what)
  {
    // If we were pressed, then exit
    if (evt.id == Event.ACTION_EVENT)
    {
      // Exit
      System.exit(0);
    }
    return true;
  }
}

class TestFrame extends Frame
{
  public TestFrame()
  {
	super("Test Window");
    resize(500, 100);
	show();
	Dlg1 ctrls = new Dlg1 (this);
    ctrls.CreateControls();
	//ctrls.m_Parent.show();
	show();
  }
}

public class TTPing implements Runnable
{
  public void run()
  {
  }

  public static void main(String args[])
  {
	// Create our window and add components to it
    Frame window = new Frame("TrackTrek '97 Ping");
	window.setBackground(new Color(192, 192, 192));
    Label status = new Label();
    Label statusLabel = new Label("Last Ping:");
    TextField host = new TextField("localhost");
    Panel panel = new Panel();
    GridBagLayout buttonLayout = new GridBagLayout();
    BorderLayout layout = new BorderLayout();
    ExitButton userExit = new ExitButton("Close", status);
    PingButton reping = new PingButton("Ping", status, host);

    status.setText("Press \"Ping\" to ping to the server...");
    window.resize(500, 100);
    window.setLayout(layout);
    panel.setLayout(buttonLayout);
    panel.add(reping);
    panel.add(userExit);
    window.add("West", statusLabel);
    window.add("Center", status);
    window.add("East", panel);
    window.add("South", host);
    window.setResizable(false);
    window.show();

	Dialog dlg = new Dialog(window, "Event Properties", false);
	dlg.setBackground(new Color(192, 192, 192));
	EventPropertiesDialog ctrls = new EventPropertiesDialog (dlg);
    ctrls.CreateControls();
	dlg.setResizable(true);
	dlg.show();
  }
}

