// TrackTrek '97 by Steven Lawrance
// Ping Utility

package sl.TrackTrek.client;

import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;

class EasyButton extends Button
{
  public boolean pressed;

  public EasyButton(String name)
  {
    super(name);
    pressed = false;
  }

  public boolean action(Event evt, Object what)
  {
    // If we were pressed, then set the pressed flag to true 
    if ((evt.id == Event.ACTION_EVENT) || (evt.id == Event.WINDOW_DESTROY))
      pressed = true;
    return true;
  }
}

public class Ping implements Runnable
{
  public final static int OK = 1;
  public final static int ERROR = 2;
  public final static int PING = 3;
  public final static int READ = 4;
  public final static int WRITE = 5;
  public final static int CLOSE = 6;
  public final static int LOCK = 7;
  public final static int NOLOCK = 9;
  public final static int NOTEXIST = 10;
  public final static int UNLOCK = 11;

  public void run()
  {
  }

  public static void main(String args[])
  {
    // Create our window and add components to it
    Frame window = new Frame("TrackTrek '97 Ping");;
    Label status = new Label();
    Label statusLabel = new Label("Last Ping:");
    Panel panel = new Panel();
    GridBagLayout buttonLayout = new GridBagLayout();
    BorderLayout layout = new BorderLayout();
    EasyButton userExit = new EasyButton("Close");
    EasyButton reping = new EasyButton("Re-Ping");
    window.resize(500, 75);
    window.setLayout(layout);
    panel.setLayout(buttonLayout);
    panel.add(reping);
    panel.add(userExit);
    window.add("West", statusLabel);
    window.add("Center", status);
    window.add("East", panel);
    window.setResizable(false);
    window.show();
    status.setText("none");
    boolean again = true;

    while (again)
    {
    try { try { try {
      // Make a connection to the local server
      status.setText("Contacting server...");
      Socket conn = new Socket("127.0.0.1", 3040);
      DataOutputStream out = new DataOutputStream(conn.getOutputStream());
      DataInputStream in = new DataInputStream(conn.getInputStream());

      // Send the ping
      out.writeInt(PING);
      status.setText("Ping sent, waiting for reply...");

      // Wait for reply
      String version;
      boolean success = true;
      success = success && (in.readInt() == OK);
      status.setText("Acknowledgement 1/2");
      Thread.sleep(500);
      success = success && (in.readInt() == PING);
      status.setText("Ping 1/3");
      Thread.sleep(500);
      success = success && (in.readInt() == PING);
      status.setText("Ping 2/3");
      Thread.sleep(500);
      success = success && (in.readInt() == PING);
      status.setText("Ping 3/3");
      Thread.sleep(500);
      version = in.readUTF();
      status.setText("Version sent");
      Thread.sleep(500);
      success = success && (in.readInt() == OK);
      status.setText("Acknowledgement 2/2");
      Thread.sleep(500);

      // Close connection
      out.writeInt(CLOSE);
      conn.close();

      // Display result
      String msg = "Reply " + ((success)? "good, ": "bad, ") + "version is \"" + version + "\"";
      status.setText(msg);
    } catch (UnknownHostException str) { status.setText("Unknown host"); } } catch (IOException str) { status.setText("Connection failure"); } } catch (InterruptedException str)
    {
      status.setText("Internal error with ping");
    }
    
    // Wait until user exits or a reping
    while (!userExit.pressed && !reping.pressed)
      Thread.yield();

    if (reping.pressed)
      reping.pressed = false;

    if (userExit.pressed)
      again = false;
    }

    // Exit
    System.exit(0);
  }
}

