// DBObject.java

package sl.TrackTrek;

import java.io.*;
import java.util.Hashtable;
import java.util.Properties;

// My own database class (stores info in individual files)
public abstract class DBObject extends Object
{
  public abstract void load();
  public abstract void save(DBObject value);

  public void delete(Hashtable table, int id)
  {
    // Remove the object; if this is the last object, decrease the maxval value
    table.remove(String.valueOf(id));
    if (id == ((Integer)table.get("maxval")).intValue())
      table.put("maxval", new Integer(id - 1));
  }

  public int getEmptySlot(Hashtable table)
  {
    return getEmptySlot(table, 0);
  }

  public int getEmptySlot(Hashtable table, int start)
  {
    // Go through and find an empty spot
    int i = start;
    while (table.containsKey(String.valueOf(i)))
      i++;
    return i;
  }

  public int add(Hashtable table, DBObject object)
  {
    // Add a value to the first free position; if it is the last position, increment maxval
    int loc = getEmptySlot(table, 0);
    if (loc > ((Integer)table.get("maxval")).intValue())
      table.put("maxval", new Integer(loc));
    table.put(String.valueOf(loc), object);
    return loc;
  }

  public InputStream getInputStream(String folder, int id)
  {
    File file = new File(folder, String.valueOf(id) + ".TrackTrek_DBObject");

    // If the directories do not exist, create them
    if (!file.isDirectory())
      file.mkdirs();

    // If the file does not exist, return null
    if (!file.exists())
      return null;

    // Everything is great!  Create the inputstream and return it
    try {
      FileInputStream in = new FileInputStream(file);
      return in;
    } catch (FileNotFoundException str) {
      return null;
    }
  }

  public OutputStream getOutputStream(String folder, int id)
  {
    File file = new File(folder, String.valueOf(id) + ".TrackTrek_DBObject");

    // If the directories do not exist, create them
    if (!file.isDirectory())
      file.mkdirs();

    // Everything is great!  Create the outputstream and return it
    try {
      FileOutputStream out = new FileOutputStream(file);
      return out;
    } catch (IOException str) {
      return null;
    }
  }

  public abstract DBObject makeNew(String name, String folder, int id);

  public Hashtable getTable(Properties prop, String name, String folder)
  {
    int maxval = Integer.valueOf(prop.getProperty(name + ".maxval", "-1")).intValue();
    Hashtable hash = new Hashtable();
    hash.put("maxval", new Integer(maxval));
    int i;
    for (i = 0; i <= maxval; i++)
      if (prop.containsKey(name + "." + String.valueOf(i)))
        hash.put(String.valueOf(i), makeNew(name, folder, i));
    return hash;
  }

  public void setTable(Hashtable table, Properties prop, String name)
  {
    int maxval = ((Integer)table.get("maxval")).intValue();
    prop.put(name + ".maxval", String.valueOf(maxval));
    int i;
    for (i = 0; i <= maxval; i++)
      if (table.containsKey(String.valueOf(i)))
        prop.put(name + "." + String.valueOf(i), table.get(String.valueOf(i)));
  }
}

