// TrackTrek date data type
// DateType.java
// By Steven Lawrance

package sl.TrackTrek;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.lang.String;
import java.util.Properties;

public class DateType extends sl.TrackTrek.DataType
{
  // Values for this object
  public int year;
  public int month;
  public int weekDay;
  public int day;
  public int hour;
  public int minute;
  public boolean includeWeekDay;
  public boolean includeTime;

  // String representations
  String monthStr[] = { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };
  String weekDayStr[] = { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" };

  // Default sort behavior
  public static boolean normallyAscend = true;

  private void createDateType(int year, int month, int day, int hour, int minute, boolean includeWeekDay)
  {
	 // Assign the variables of this object
	 this.year = year;
	 this.month = month;
	 this.day = day;
	 this.hour = hour;
	 this.minute = minute;
	 this.includeWeekDay = includeWeekDay;

	 // Determine the day of the week
         GregorianCalendar weekDay = new GregorianCalendar(year, month, day);
         weekDay.complete();
         this.weekDay = weekDay.get(Calendar.DAYOFWEEK) - Calendar.SUNDAY;

	 // Set flags
	 includeTime = true;
  }

  public DateType(int year, int month, int day, int hour, int minute, boolean includeWeekDay)
  {
	 // Assign the new date variables
      super();
	 createDateType(year, month, day, hour, minute, includeWeekDay);
  }

  public DateType(int year, int month, int day, int hour, int minute)
  {
	 // Programmer didn't specify the weekDay flag; assume false
      super();
	 createDateType(year, month, day, hour, minute, false);
  }

  public DateType(int year, int month, int day)
  {
	 // Programmer is too lazy to add two zeros, so I must do it
      super();
	 createDateType(year, month, day, 0, 0, false);
	 includeTime = false;
  }

  public DateType()
  {
	 // Programmer was REALLY lazy!!  It is now January 1, 1997
      super();
	 createDateType(1997, 0, 0, 0, 0, false);
	 includeTime = false;
  }

  public String toString()
  {
	 // Convert myself into a string
         return ((includeWeekDay)? weekDayStr[weekDay] + ", ": "") + monthStr[month] +
                 " " + java.lang.String.valueOf(day + 1) + ", " + java.lang.String.valueOf(year) + 
                 ((includeTime)? "  " + java.lang.String.valueOf((hour == 0)? 12: (hour + 1) % 12) + ":" + java.lang.String.valueOf(minute + 1) +
                 ((hour >= 12)? "PM": "AM"): "");
  }

  public void setValue(long value)
  {
	 // Set the values
         year = (int) value / (13 * 32 * 25 * 61);
         value = value % (13 * 32 * 25 * 61);

         month = (int) value / (32 * 25 * 61);
         value = value % (32 * 25 * 61);

         day = (int) value / (25 * 61);
         value = value % (25 * 61);

         hour = (int) value / 61;
         value = value % 61;

	 minute = (int) value;


	 // Determine the day of the week
         GregorianCalendar weekDay = new GregorianCalendar(year, month, day + 1);
         weekDay.complete();
         this.weekDay = weekDay.get(Calendar.DAYOFWEEK) - Calendar.SUNDAY;
  }

  public static long getSetValue(int year, int month, int day, int hour, int minute)
  {
	 // Convert ourselves into minutes for the setValue() function
	 long minutes;
         minutes = year * 13 * 32 * 25 * 61;
         minutes += month * 32 * 25 * 61;
         minutes += day * 25 * 61;
         minutes += hour * 61;
	 minutes += minute;
	 return minutes;
  }

  public int compareTo(DataType other)
  {
	 // Get the number of minutes in both dates
	 long otherValue = other.getLongValue();
	 long thisValue = this.getLongValue();

	 // This time is less than the other time
	 if (otherValue > thisValue)
		return -1;

	 // This time equals the other time
	 if (otherValue == thisValue)
		return 0;

	 // This time is greater than the other time
	 return 1;
  }

  public long getLongValue()
  {
	 // Convert ourselves into minutes; only use this result with DateType functions!!
	 long minutes;
	 minutes = year * 12 * 31 * 24 * 60;
	 minutes += month * 31 * 24 * 60;
	 minutes += day * 24 * 60;
	 minutes += hour * 60;
	 minutes += minute;
	 return minutes;
  }

  public String getStringValue()
  {
         return String.valueOf(getSetValue(year, month, day, hour, minute));
  }

  public void setValue(String value)
  {
	 long lValue = java.lang.Long.parseLong(value);
	 setValue(lValue);
  }

  public void loadProperties(Properties prop, String name)
  {
    year = Integer.getInteger(prop.getProperty(name + ".year", "0")).intValue();
    month = Integer.getInteger(prop.getProperty(name + ".month", "0")).intValue();
    day = Integer.getInteger(prop.getProperty(name + ".day", "0")).intValue();
    weekDay = Integer.getInteger(prop.getProperty(name + ".weekDay", "0")).intValue();
    hour = Integer.getInteger(prop.getProperty(name + ".hour", "0")).intValue();
    minute = Integer.getInteger(prop.getProperty(name + ".minute", "0")).intValue();
    includeWeekDay = Boolean.getBoolean(prop.getProperty(name + ".includeWeekDay", "true"));
    includeTime = Boolean.getBoolean(prop.getProperty(name + ".includeTime", "true"));
    normallyAscend = Boolean.getBoolean(prop.getProperty(name + ".normallyAscend", "true"));
  }

  public void saveProperties(Properties prop, String name)
  {
    prop.put(name + ".year", String.valueOf(year));
    prop.put(name + ".month", String.valueOf(month));
    prop.put(name + ".day", String.valueOf(day));
    prop.put(name + ".weekDay", String.valueOf(weekDay));
    prop.put(name + ".hour", String.valueOf(hour));
    prop.put(name + ".minute", String.valueOf(minute));
    prop.put(name + ".includeWeekDay", String.valueOf(includeWeekDay));
    prop.put(name + ".includeTime", String.valueOf(includeTime));
    prop.put(name + ".normallyAscend", String.valueOf(normallyAscend));
  }
}

