// TrackTrek distance data type
// Distance.java
// By Steven Lawrance

package sl.TrackTrek;
import java.lang.String;
import java.util.Properties;

public class Distance extends sl.TrackTrek.DataType
{
  // Values for this object
  private long english;  // in 1/16 inches
  private long metric;   // in millimeters
  public boolean useMetric;

  // Display flags
  public boolean includeHuge;   // kilometers and miles
  public boolean includeLarge;  // meters and yards
  public boolean includeMedium; // decimeters and feet
  public boolean includeSmall;  // centimeters and inches
  public boolean includeTiny;   // millimeters and fractions of inches

  // Default sort behavior
  public static boolean normallyAscend = false;

  public Distance(long distance, boolean useMetric)
  {
	 // Set this distance's value to the given distance and set the system
      super();
	 this.useMetric = useMetric;
	 this.metric = (useMetric)? distance: metricValue(distance);
	 this.english = (useMetric)? englishValue(distance): distance;
	 includeHuge = false;
	 includeLarge = true;
	 includeMedium = (useMetric)? false: true;
	 includeSmall = true;
	 includeTiny = false;
  }

  public Distance(long distance)
  {
	 // Set this distance's value to the given distance and use metric
      super();
	 this.useMetric = true;
	 this.metric = (useMetric)? distance: metricValue(distance);
	 this.english = (useMetric)? englishValue(distance): distance;
	 includeHuge = false;
	 includeLarge = true;
	 includeMedium = (useMetric)? false: true;
	 includeSmall = true;
	 includeTiny = false;
  }

  public Distance()
  {
	 // Set this distance's value to zero and assume metric
      super();
	 this.useMetric = true;
	 this.metric = 0;
	 this.english = 0;
	 includeHuge = false;
	 includeLarge = true;
	 includeMedium = (useMetric)? false: true;
	 includeSmall = true;
	 includeTiny = false;
  }

  public long getValue(boolean getMetric)
  {
	 // Return the desired variable
         return (getMetric)? metric: english;
  }

  public void setValue(long distance, boolean useMetric)
  {
	 // Set this distance's value to the given distance and set the current system
	 this.useMetric = useMetric;
	 this.metric = (useMetric)? distance: metricValue(distance);
	 this.english = (useMetric)? englishValue(distance): distance;
  }

  public void setValue(long distance)
  {
	 // Set this distance's value to the given distance in the current system
	 this.metric = (useMetric)? distance: metricValue(distance);
	 this.english = (useMetric)? englishValue(distance): distance;
  }

  public long englishValue(long metric)
  {
	 // Convert millimeters into 1/16 inches
         return (long) (metric / 1.5875);
  }

  public long metricValue(long english)
  {
	 // Convert 1/16 inches into millimeters
         return (long) (english * 1.5875);
  }

  public String toString()
  {
	 // Convert myself into a string
	 if (useMetric)
	 {
		// Display with the metric system
		String output = new String();
                long distance = metric;

		if (includeHuge)
		{
                  output = output.concat(java.lang.String.valueOf(distance / 1000000) + "km ");
                  distance = distance % 1000000;
		}

		if (includeLarge)
		{
                 output =  output.concat(java.lang.String.valueOf(distance / 1000) + "m ");
                  distance = distance % 1000;
		}

		if (includeMedium)
		{
                  output = output.concat(java.lang.String.valueOf(distance / 100) + "dm ");
                  distance = distance % 100;
		}

		if (includeSmall)
		{
                  output = output.concat(java.lang.String.valueOf(distance / 10) + "cm ");
                  distance = distance % 10;
		}

		if (includeTiny)
                  output = output.concat(java.lang.String.valueOf(distance) + "mm");

		// Remove the ending space (if any)
                output = output.trim();

		return output;
	 }
	 else
	 {
		// Display with the english system
		String output = new String();
                long distance = english;

		if (includeHuge)
		{
                  output = output.concat(java.lang.String.valueOf(distance / 1013760) + "mi ");
                  distance = distance % 1013760;
		}

		if (includeLarge)
		{
                  output = output.concat(java.lang.String.valueOf(distance / 576) + "yd ");
                  distance = distance % 576;
		}

		if (includeMedium)
		{
                  output = output.concat(java.lang.String.valueOf(distance / 192) + "'");
                  distance = distance % 192;
		}

		if (includeSmall)
		{
                  output = output.concat(java.lang.String.valueOf(distance / 16) + ((!includeTiny)? "\"": ""));
                  distance = distance % 16;
		}

		if (includeTiny)
		{
		  if (distance != 0)
		  {
			 // Write a simplified fraction
			 if (distance % 8 == 0)
                                output = output.concat(" 1/2\"");
			 else
				if (distance % 4 == 0)
                                  output = output.concat(" " + java.lang.String.valueOf(distance / 4) + "/4\"");
				else
				  if (distance % 2 == 0)
                                         output = output.concat(" " + java.lang.String.valueOf(distance / 2) + "/8\"");
				  else
                                         output = output.concat(" " + java.lang.String.valueOf(distance) + "/16\"");
		  }
		  else
			 if (includeSmall)
                                output = output.concat("\"");
		}

		// Remove the ending space (if any)
                output = output.trim();

		return output;
	 }
  } /* toString */

  public int compareTo(DataType other)
  {
	 // Get the number of minutes in both dates
	 long otherValue = other.getLongValue();
	 long thisValue = this.getLongValue();

	 // This time is less than the other time
	 if (otherValue > thisValue)
		return -1;

	 // This time equals the other time
	 if (otherValue == thisValue)
		return 0;

	 // This time is greater than the other time
	 return 1;
  }

  public String getStringValue()
  {
         return String.valueOf((useMetric)? metric: english);
  }

  public void setValue(String value)
  {
	 long lValue = java.lang.Long.parseLong(value);
	 setValue(lValue);
  }

  public long getLongValue()
  {
	 // Return the desired variable in the current system
         return (useMetric)? metric: english;
  }

  public void loadProperties(Properties prop, String name)
  {
    metric = Long.getLong(prop.getProperty(name + ".metric", "0")).longValue();
    english = Long.getLong(prop.getProperty(name + ".english", "0")).longValue();
    useMetric = Boolean.getBoolean(prop.getProperty(name + ".useMetric", "true"));
    includeHuge = Boolean.getBoolean(prop.getProperty(name + ".includeHuge", "false"));
    includeLarge = Boolean.getBoolean(prop.getProperty(name + ".includeLarge", "true"));
    includeMedium = Boolean.getBoolean(prop.getProperty(name + ".includeMedium", ((useMetric)? "false": "true")));
    includeSmall = Boolean.getBoolean(prop.getProperty(name + ".includeSmall", "true"));
    includeTiny = Boolean.getBoolean(prop.getProperty(name + ".includeTiny", "false"));
    normallyAscend = Boolean.getBoolean(prop.getProperty(name + ".normallyAscend", "false"));
  }

  public void saveProperties(Properties prop, String name)
  {
    prop.put(name + ".metric", String.valueOf(metric));
    prop.put(name + ".english", String.valueOf(english));
    prop.put(name + ".includeHuge", String.valueOf(includeHuge));
    prop.put(name + ".includeLarge", String.valueOf(includeLarge));
    prop.put(name + ".includeMedium", String.valueOf(includeMedium));
    prop.put(name + ".includeSmall", String.valueOf(includeSmall));
    prop.put(name + ".includeTiny", String.valueOf(includeTiny));
    prop.put(name + ".normallyAscend", String.valueOf(normallyAscend));
  }
}

