// TrackTrek time data type
// TimeType.java
// By Steven Lawrance

package sl.TrackTrek;
import java.lang.*;
import java.util.Properties;

public class TimeType extends sl.TrackTrek.DataType
{
  // Values for this object
  public long value;

  // Display variables
  public boolean includeHours;
  public boolean includeMinutes;
  public boolean includeSeconds;
  public boolean includeCentiseconds;

  // Default sort behavior
  public static boolean normallyAscend = false;

  public TimeType(long value)
  {
	 // Set this time to the specified value
      super();
	 this.value = value;
	 includeHours = false;
	 includeMinutes = true;
	 includeSeconds = true;
	 includeCentiseconds = true;
  }

  public TimeType()
  {
	 // Set this time to zero values
      super();
	 this.value = 0;
	 includeMinutes = true;
	 includeSeconds = true;
	 includeCentiseconds = true;
  }

  public String toString()
  {
	 // Convert myself into a string
         String outp = new String();

         long time = this.value;
	 if (includeHours)
	 {
                outp = outp.concat(java.lang.String.valueOf((long) (time / 360000)) + ((includeMinutes)? ":": "hrs "));
                time = time % 360000;
	 }

	 if (includeMinutes)
	 {
                outp = outp.concat((((time / 6000 < 10) && (includeHours))? "0": "") + java.lang.String.valueOf((long) (time / 6000)) + ((includeSeconds)? ":": "min "));
                time = time % 6000;
	 }

	 if (includeSeconds)
	 {
                outp = outp.concat((((time / 100 < 10) && (includeMinutes))? "0": "") + java.lang.String.valueOf((long) (time / 100)));
                time = time % 100;
	 }

	 if (includeCentiseconds)
                outp = outp.concat("." + (((time < 10) && (includeSeconds))? "0": "") + java.lang.String.valueOf(time));

	 // Remove the ending space (if any)
         outp = outp.trim();

         return outp;
  }

  public int compareTo(DataType other)
  {
	 // Get the number of minutes in both dates
	 long otherValue = other.getLongValue();
	 long thisValue = this.getLongValue();

	 // This time is less than the other time
	 if (otherValue > thisValue)
		return -1;

	 // This time equals the other time
	 if (otherValue == thisValue)
		return 0;

	 // This time is greater than the other time
	 return 1;
  }

  public long getLongValue()
  {
	 return value;
  }

  public String getStringValue()
  {
         return String.valueOf(value);
  }

  public void setValue(long value)
  {
	 // Set the value
	 this.value = value;
  }

  public void setValue(String value)
  {
	 this.value = java.lang.Long.parseLong(value);
  }

  public void loadProperties(Properties prop, String name)
  {
    value = Long.getLong(prop.getProperty(name + ".value", "0")).longValue();
    includeHours = Boolean.getBoolean(prop.getProperty(name + ".includeHours", "false"));
    includeMinutes = Boolean.getBoolean(prop.getProperty(name + ".includeMinutes", "true"));
    includeSeconds = Boolean.getBoolean(prop.getProperty(name + ".includeSeconds", "true"));
    includeCentiseconds = Boolean.getBoolean(prop.getProperty(name + ".includeCentiseconds", "true"));
    normallyAscend = Boolean.getBoolean(prop.getProperty(name + ".normallyAscend", "false"));
  }

  public void saveProperties(Properties prop, String name)
  {
    prop.put(name + ".value", String.valueOf(value));
    prop.put(name + ".includeHours", String.valueOf(includeHours));
    prop.put(name + ".includeMinutes", String.valueOf(includeMinutes));
    prop.put(name + ".includeSeconds", String.valueOf(includeSeconds));
    prop.put(name + ".includeCentiseconds", String.valueOf(includeCentiseconds));
    prop.put(name + ".normallyAscend", String.valueOf(normallyAscend));
  }
}

