package sl.TrackTrek ;

import java.lang.*;
import w3c.jigsaw.http.*;
import w3c.jigsaw.resources.*;

import w3c.jigsaw.html.* ;
import w3c.jigsaw.forms.*;

/**
 * Page handler routines.
 */

public class Page {

	/**
	 * Create a new page with the standard background and font colors.
	 * @param title The title of the web page.  This appears on the title bar.
	 * @param heading The heading is centered in large letters on the top, if not null.
	 */
	public static void header(HtmlGenerator html, String heading)
	{
	  html.emitBODYTag(false);
	  html.meta("GENERATOR", "TrackTrek 1.02");
	  html.meta("VERSION", "1.02");
	  html.meta("PRODUCT", "TrackTrek");
	  html.meta("AUTHOR", "Steven Lawrance");
	  html.meta("COPYRIGHT", "Copyright (c) 1996, Steven Lawrance");
	  html.append("<body bgcolor=\"#0080FF\" text=\"#FFFFFF\" link=\"#FFFF00\" vlink=\"#FFFF00\">");
	  if (heading != "")
		 html.append("<h1 align=center><font color=\"#FFFFFF\"><font size=7>" + heading + "</font></font></h1>");
	}

	public static void sort(String content[][], int start, int end, int c, int sortColumn, int columns)
	{
	  int index, inner, r;

	  // Sort the current column
	  for (r = start; r <= end - 1; r++)
	  {
		 index = r;
		 for (inner = r + 1; inner <= end; inner++)
			if ((content[inner - 1][c - 1].toLowerCase()).compareTo(content[index - 1][c - 1].toLowerCase()) < 0)
			  index = inner;
		 if (index != r)
		 {
			String temp[] = content[index - 1];
			content[index - 1] = content[r - 1];
			content[r - 1] = temp;
		 }
	  }

	  // Find any duplicates
	  String prev = content[start - 1][c - 1];
	  int begin = start;
	  boolean same = false;
	  for (r = start + 1; r <= end; r++)
	  {
		 if (content[r - 1][c - 1] == prev)
		 {
			if (!same)
			{
			  begin = r - 1;
			  same = true;
			}
			if (same && (r == end))
			{
			  if (sortColumn == c)
			  {
				 if (sortColumn != 1)
					sort(content, begin, r, 1, sortColumn, columns);
				 if ((sortColumn == 1) && (columns > 1))
					sort(content, begin, r, 2, sortColumn, columns);
			  }
			  else
			  {
				 if (c + 1 == sortColumn)
				 {
					if (c + 2 <= columns)
					  sort(content, begin, r, c + 2, sortColumn, columns);
				 }
				 if ((c + 1 != sortColumn) && (c + 1 <= columns))
					sort(content, begin, r, c + 1, sortColumn, columns);
			  }
			}
		 }
		 else
		 {
			if (same)
			{
			  if (sortColumn == c)
			  {
				 if (sortColumn != 1)
					sort(content, begin, r - 1, 1, sortColumn, columns);
				 if ((sortColumn == 1) && (columns > 1))
					sort(content, begin, r - 1, 2, sortColumn, columns);
			  }
			  else
			  {
				 if (c + 1 == sortColumn)
				 {
					if (c + 2 <= columns)
					  sort(content, begin, r - 1, c + 2, sortColumn, columns);
				 }
				 if ((c + 1 != sortColumn) && (c + 1 <= columns))
					sort(content, begin, r - 1, c + 1, sortColumn, columns);
			  }
			  same = false;
			}
			prev = content[r - 1][c - 1];
		 }
	  }
	}

	/**
	 * Display a table.
	 * @param html The HtmlGenerator variable.
	 * @param content A 2-dimensional array that contains the table: content[row][column].
	 * @param label The labels at the top of the columns.
	 * @param rows The number of rows.
	 * @param columns The number of columns.
	 * @param sortName Execute this when the sort button is pressed.
	 * @param sortColumn The column to sort by.  Any duplicates are sorted by the first column, then the second, and so forth until sorted.
	 * @param actionName Execute this when the action button is pressed (if exists).  Specify null for no action button.
	 * @param actionButton The text to display on the action button, if actionName is not null.
	 */

	public static void table(HtmlGenerator html, String content[][], String label[], int rows, int columns, String sortName, int sortColumn, String actionName, String actionButton)
	{
	  // Calculate width variables and store them as strings (for HTML purposes)
	  String width = new String(String.valueOf(100 / columns));
	  String halfWidth = new String(String.valueOf(100 / (columns * 2)));
	  int r, c;

	  // Write table header
	  html.append("<hr>\n");
	  html.append("<form action=\"" + sortName + "\" method=\"GET\">\n");
	  html.append("<table cellpadding=0 cellspacing=1 width=100%>\n<tr>");
	  for (c = 1; c <= columns; c++) {
		 html.append("<td valign=bottom width=" + ((c == columns)? halfWidth: width) + "%><input type=radio" + ((c == sortColumn)? " checked": ""));
		 html.append(" name=\"SortOrder\" value=\"" + String.valueOf(c) + "\"><font size=4>" + label[c - 1] + "</font></td>");
	  }
	  html.append("<td align=right valign=bottom width=" + halfWidth);
	  html.append("%><input type=submit value=\"Sort List\">");
	  html.append("</td></tr></form>\n");

	  // Write horizontal line
	  html.append("<tr><td valign=top colspan=4 width=100%>");
	  html.append("<hr size=1>\n</td></tr>\n");

	  // Sort List (this sorts the list and sorts any duplicates from the order 1 < sortColumn < columns
	  sort(content, 1, rows, sortColumn, sortColumn, columns);

	  // Write form data (if form was requested)
	  if (actionName != null)
		 html.append("<form action=\"" + actionName + "\" method=\"GET\">\n");

	  // Write table contents
	  for (r = 1; r <= rows; r++)
	  {
		 html.append("<tr>");
		 for (c = 1; c <= columns; c++)
			html.append("<td valign=top" + ((c == columns)? " colspan=2>": ">") + content[r - 1][c - 1] + "</td>");
		 html.append("</tr>\n");
	  }
	  html.append("</table>\n");

	  // Write form data (if form was requested)
	  if (actionName != null)
		 html.append("<br><br>\n<input type=submit value=\"" + actionButton + "\"></form>\n");

	  // Write ending horizontal line
	  html.append("<hr>\n");
	}
}

